import java.awt.Container;
import java.util.ArrayList;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JTextField;

/*
 * This program illustrates working with an ArrayList of textfields, an array
 * of ints, and the use of a lambda expression to implement an ActionListener.
 * Text's constructor takes an int array of any length, creates a matching number
 * of textfields and sets their initial values to the array's values.  The
 * ActionListener associated with each textfield doesn't do anything particularly
 * useful.  It's just an example.
 * 
 *  As the Text class extends Box, all methods implemented in the Box class are
 *  available for use.  Note that Box's constructor, invoked by the super call
 *  in Text's constructor, MUST be called first.  super must always be called
 *  first when using a superclass' constructor.
 */

public class Text extends Box {
	
	private int[] data;
	// The textfields are kept here, so that we can get at them later to 
	// compute their sum.
	private ArrayList<JTextField> tfs = new ArrayList<JTextField>();
	
	public Text(int[] data) {
		//The superclass' constructor MUST be called first. 
		super(BoxLayout.Y_AXIS);
		this.data = data;
		
		// Process each element of the array.
		for (int i = 0; i < data.length; i++) {
			// Because i's value is changing, it can't be used in the lambda
			// expression.  So, let's create a final int to use i's value
			// and make the Java compiler happy by using it in the lambda expression
			final int index = i;
			JTextField tf = new JTextField(20);  // Each textfield is 20 chars. wide
			tf.setText(data[i] + "");
			// This adds tf to the _Box_ so that it's displayed as part of the
			// window.
			add(tf);
			// This adds tf to the _ArrayList_ so that it's available later
			// to compute the sum of the textfields
			tfs.add(tf);
			
			// ActionListener is a functional interface, with just one method,
			// actionPerformed, that takes a single parameter, e, the ActionEvent.
			// It's a good candidate for a lambda expression implementation.
			// This ActionListener doesn't do anything particularly important; it's
			// just an illustration.
			
			tf.addActionListener(e -> {
				// Print the textfield's index in the ArrayList, the textfield's
				// current value, and its original value.
				System.out.println("Index " + index + ", value " +
						Integer.parseInt(tf.getText()) + ", original value " +
						data[index]);
				
				// Compute and print the sum of the textfields.
				
				int sum = 0;
				for (int j = 0; j < tfs.size(); j++)
					sum += Integer.parseInt(tfs.get(j).getText());
				System.out.println("Sum " + sum);
			});
		}
	}

	public static void main(String[] args) {
		JFrame window = new JFrame("Text Example");
		Container pane = window.getContentPane();
		// The data array we're using.  It can contain as many, or few, elements
		// as you wish.
		int[] data = {1, 2, 3, 42};
		Text t = new Text(data);
		pane.add(t);
		window.pack();
		window.setLocation(500, 200);
		window.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		window.setVisible(true);
	}
}
